// =======================================================================
//
//                     <IV-Win/MWlib.c>
//
//  General definitions for the MW library, a library implementing an 
//  object oriented structure on top of MS-Windows.
//
//
// $Revision: 1.2 $
// $Date: 1997/03/31 16:45:06 $
//
// Windows 3.1/NT Port 
// Copyright (c) 1993 Tim Prinzing
//
// This media contains programs and data which are proprietary
// to Tim Prinzing.
//
// These contents are provided under a Tim Prinzing software source
// license, which prohibits their unauthorized resale or distribution 
// outside of the buyer's organization.
// 
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
//
// IN NO EVENT SHALL Tim Prinzing BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER 
// RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE 
// POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR 
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//
//  Author:		Tim Prinzing
//  System:		MS-Windows 3.x
//  Date:		1-24-93
//
//  Copyright (c) 1993
// =======================================================================

// ---- local includes ----
#include <IV-Win/MWapp.h>
#include <OS/string.h>
#include <InterViews/style.h>

#ifndef WIN32
#include <MMgAlloc.h>
#endif

// ---- ansi/posix includes ----
#include <stdio.h>
#include <stdlib.h>

// ----------------------------------------------------------------------
// Visual assertion message.  This is provided in response to the 
// failure of an assertion test... ie it's a bug trap.
// ----------------------------------------------------------------------
void mwAssertion(
	const char* msg, 					// message to display
	const char* file, 					// file where it happened
	unsigned int line)					// line where it happened
{
	char buff[135];
	sprintf(buff,"at line %u, file %s: `%s'", line, file, msg);
	MessageBeep(0);
	if (MessageBox(0, buff, "Assertion Failed", 
		MB_OKCANCEL | MB_ICONSTOP | MB_TASKMODAL) != IDOK)
	{
		exit(1);
	}
}

#ifndef WIN32
// ----------------------------------------------------------------------
// global new and delete operators
// ----------------------------------------------------------------------

MMgpAlloc allocator;

void* operator new(size_t size)
{
#ifdef __LARGE__
	return allocator.alloc(size);
#else
	NOT SUPPORTED
#endif
}

void operator delete(void* ptr)
{
	if (ptr)
    {
#ifdef __LARGE__
		allocator.free(ptr);
#else
		NOT SUPPORTED
#endif
	}
}

#endif /* WIN32 */

// --------------------------------------------------------------------
// style debug stuff.  
// --------------------------------------------------------------------
void dumpStyle(Style* s, const char* path) 
{
	const NullTerminatedString nm(*s->name());	// style name
	long i;										// tmp index
	char buff[512];

	// ---- load the attributes ----
	long nattr = s->attribute_count();
	for (i = 0; i < nattr; i++)
	{
		String an;
		String av;
		if (s->attribute(i, an, av))
		{
			NullTerminatedString aname(an);
			NullTerminatedString avalue(av);

			const char* adjusted = aname.string();
			while (*(adjusted) == '*')
				adjusted++;
			sprintf(buff,"%s*%s: %s\n", path, adjusted, avalue.string());
			OutputDebugString(buff);
		}
	}

	// ---- do all child styles ----
	long nchild = s->children();
	for (i = 0; i < nchild; i++)
	{
		Style* c = s->child(i);
		const NullTerminatedString cnm(*c->name());	// style name
		if (c)
		{
			sprintf(buff,"%s*%s", path, cnm.string());
			dumpStyle(c, buff);
		}
	}
}


